import logging

from my_deeptools_win._counter import multi_region_file_BAM_count
from my_deeptools_win.__logging import set_logging_level
from my_deeptools_win._spliter import  split_file_and_make_temp
def main(input,
            bams: str,
            tags,
            output: str=None,
            method: str="CPM",
            process: int =1,
            mapq: int = 20,
            strand: str = "b",
            extend_length : int = 1,
            temp_dir: str = None,
            verbose: str = "INFO",):
    set_logging_level(verbose)
    bam_file_list = bams.split(",") if not isinstance(bams, tuple) else list(bams)
    bam_tags_list = tags.split(",") if not isinstance(tags, tuple) else list(tags)
    assert len(bam_file_list) == len(bam_tags_list), "bam file list and tags list should have same length"
    print(bam_file_list)
    print(bam_tags_list)
    # split bed file

    region_split_filename_list = split_file_and_make_temp(input_file_name = input,
                                                           n_part = process,
                                                           temp_dir = temp_dir)
    #logging.info("split bed file")
    # count file

    final_count_dict = {}
    tag_res_list = []

    for bam_index, run_bam_filename in enumerate(bam_file_list):
        bam_tag = bam_tags_list[bam_index]
        count_list = multi_region_file_BAM_count(
            region_filename_list=region_split_filename_list,
            bam_filename=run_bam_filename,
            thread_num=process,
            MAPQ_cutoff=mapq,
            strand_select_method=strand,
            extend_length=extend_length,
            norm_method=method
        )
        # dict_key = "%s_%s" % (bam_tag,bam_index)
        # final_count_dict[dict_key] = count_list
        # tag_res_list.append(dict_key)
        
if __name__ == '__main__':
    main(1,2)


from time import sleep
from rich.console import Console

console = Console()
tasks = [f"task {n}" for n in range(1, 11)]

with console.status("[bold green]Working on tasks...") as status:
    while tasks:
        task = tasks.pop(0)
        sleep(1)
        console.log(f"{task} complete")


import logging
from multiprocessing import Pool
import bamnostic as bs
import os

def get_BAM_total_align_count(bam_filename):
    """
    INPUT:
        <bam_filename>
            BAM file path with .bai index file in a same dir

    RETURN
        <total_align_count>
            int, BAM file total align count

    """
    bam_file = bs.AlignmentFile(bam_filename, "rb")
    #bam_file.ge
    total_align_count = 0

    for bam_index_info in bam_file.get_index_stats():
        # ref_name = bam_index_info[0]
        map_count = bam_index_info[0]

        total_align_count += map_count

    bam_file.close()

    return total_align_count

def query_region_align_count(
        bam_obj,
        region_chr,
        region_start,
        region_end,
        strand_info=".",
        MAPQ_cutoff=20,
        strand_select_method="b",
        extend_length=1):
    query_align_count = 0

    for align in bam_obj.fetch(contig=region_chr,
                               start=region_start - extend_length,
                               end=region_end + extend_length):

        if align.mapq < MAPQ_cutoff:
            continue

        if strand_select_method == "s":
            if align.is_reverse and strand_info == "+":
                continue

            if not align.is_reverse and strand_info == "-":
                continue

        elif strand_select_method == "f":
            if align.is_reverse:
                continue

        elif strand_select_method == "r":
            if not align.is_reverse:
                continue

        elif strand_select_method == "b":
            pass

        else:
            raise ValueError("<strand_select_method> should be s, f, r or b!")

        query_align_count += 1

    return query_align_count

def region_file_BAM_count(
        region_filename,
        bam_filename,
        MAPQ_cutoff=20,
        strand_select_method="b",
        extend_length=1,
        norm_method="CPM"
):
    region_base_filename = os.path.basename(region_filename)
    bam_base_filename = os.path.basename(bam_filename)
    scale_factor = 1.0

    if norm_method == "CPM" or norm_method == "RPKM":
        bam_total_align_count = get_BAM_total_align_count(bam_filename)
        scale_factor = bam_total_align_count / 1e6
    print(bam_total_align_count)
    region_file = open(region_filename,"r")
    bam_file = bs.AlignmentFile(bam_filename,"rb")

    region_count_list = []
    for index,line in enumerate(region_file):
        if index % 1000 == 0:
            logging.info("Processed %d regions" % index)

        line_list = line.strip().split("\t")
        query_chr_name = line_list[0]
        query_start = int(line_list[1])
        query_end = int(line_list[2])

        if line_list[5] in ["+", "-","."]:
            query_strand_info = line_list[5]
        else:
            query_strand_info = "."

        query_count = query_region_align_count(
            bam_obj=bam_file,
            region_chr=query_chr_name,
            region_start=query_start,
            region_end=query_end,
            strand_info=query_strand_info,
            MAPQ_cutoff=MAPQ_cutoff,
            strand_select_method=strand_select_method,
            extend_length=extend_length
        )
        region_count_list.append(query_count * scale_factor)
    bam_file.close()
    region_file.close()
    return region_count_list
def multi_region_file_BAM_count(
        region_filename_list,
        bam_filename,
        thread_num=1,
        MAPQ_cutoff=20,
        strand_select_method="b",
        extend_length=1,
        norm_method="CPM"):
    logging.info("Starting multi_region_file_BAM_count")
    pool = Pool(processes=thread_num)

    input_BAM_count_result = []
    for query_region_filename in region_filename_list:
        input_BAM_count_result.append(
            pool.apply_async(region_file_BAM_count,
                             args=(query_region_filename,
                                   bam_filename,
                                   MAPQ_cutoff,
                                   strand_select_method,
                                   extend_length,
                                   norm_method))
        )
    pool.close()
    pool.join()

    merge_total_count_list = []
    for temp_index,res in enumerate(input_BAM_count_result):
        run_res = res.get()
        print(run_res)
        merge_total_count_list += run_res

    logging.info("Finished multi_region_file_BAM_count")
    print(merge_total_count_list)
    return merge_total_count_list


import logging
from logging import CRITICAL,ERROR,WARNING,INFO,DEBUG,NOTSET

def set_logging_level(level="NOTSET"):

    level = level.upper()
    print(f"set logging level = {level}")
    dt_level = {
        k:v
        for k,v in zip(["CRITICAL","ERROR","WARNING","INFO","DEBUG","NOTSET"],
                      [CRITICAL,ERROR,WARNING,INFO,DEBUG,NOTSET])
    }
    logging.basicConfig(level=dt_level[level])

if __name__=='__main__':
    set_logging_level()
    logging.info("hello world")
    logging.debug("hello world")
    logging.warning("hello world")
    logging.error("hello world")
    logging.critical("hello world")


from fire import Fire
from my_deeptools_win.count_use_bed import main as run_count_use_bed
from my_deeptools_win.multi_test import  main as convert_bam_to_bw
class Cli(object):
    """Cli interface of python package <deep_tools>

    """
    def count_use_bed(
            self,
            input,
            bams: str,
            tags,
            output: str=None,
            method: str="CPM",
            process: int =1,
            mapq: int = 20,
            strand: str = "b",
            extend_length : int = 1,
            temp_dir: str = None,
            verbose: str = "ERROR",
        ):
        """

        :param input:
        :param bams:
        :param output:
        :param method:
        :param process:
        :param mapq:
        :param strand:
        :param extend_length:
        :param temp_dir:
        :param verbose:
        :return:
        """
        return run_count_use_bed(
            input = input,
            bams = bams,
            output = output,
            tags = tags,
            method = method,
            process = process,
            mapq = mapq,
            strand = strand,
            extend_length = extend_length,
            temp_dir = temp_dir,
            verbose = verbose
        )
    def bamcovrrage(self,
                    bam_file_path,
                    bw_filw_path,
                    genome_dt,
                    threads=1,
                    binsize=10,
                    verbose="INFO",
                    tmp_path="./test_random.bigbed",
                    scale_method="cpm"
                    ):
        """
        convert bam file to bigwig file,
        :param bam_file_path: bam file path
        :param bw_file_path: out put bigwig file path
        :param genome_dt: genome length file ,such as .fai file
        :param threads: threads number
        :param binsize: genome binsize
        :param verbose:
        :param tmp_path: tmp bigbed file path
        :param scale_method: scale method function include rpkm ,cpm and none
        :return:
        """
        return convert_bam_to_bw(
            bam_file_path,
            bw_filw_path,
            genome_dt,
            threads=threads,
            binsize=binsize,
            verbose=verbose,
            tmp_path = tmp_path,
            scale_method=scale_method
        )

def main():
    cli = Cli()
    Fire(cli, name='my_deeptools_win')

if __name__ == '__main__':
    main()


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['my_deeptools_win']

package_data = \
{'': ['*']}

install_requires = \
['bamnostic>=1.1.10,<2.0.0',
 'fire>=0.6.0,<0.7.0',
 'matplotlib>=3.9.2,<4.0.0',
 'rich>=13.9.2,<14.0.0']

entry_points = \
{'console_scripts': ['deeptools_win = my_deeptools_win._cli:main']}

setup_kwargs = {
    'name': 'my-deeptools-win',
    'version': '0.1.2',
    'description': 'deeptools on windows',
    'long_description': '## *deeptools in wondows*\n\nbased on bamnostic and pybigtools\n-->\n## :writing_hand: xpf\n\n[![pipy](https://img.shields.io/badge/PIPY-0.1.1-green)](https://pypi.org/project/my-deeptools-win/)\n[![github](https://img.shields.io/badge/GITHUB-0.1.1-green)](https://github.com/yang)\n[![windows](https://img.shields.io/badge/WINDOWS-11-blue)](https://www.microsoft.com/zh-cn/windows/windows-11)\n[![bamnostic](https://img.shields.io/badge/BAMNOSTIC-1.0.0-blue)](https://pypi.org/project/bamnostic/)',
    'author': 'xpf10',
    'author_email': '2018303010006@whu.edu.cn',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



import time
import os
from multiprocessing import Process
import threading
from threading import current_thread
def task_p(x):
    time.sleep(2)
    print(f"Sub processes - {os.getpid()}: task - {x}: Result - {2**x})")

def run_single_process():
    print("**** single process ****")
    print(f"main process - {os.getpid()}")
    start = time.time()
    for i in range(10):
        task_p(i)
    end = time.time()
    print(f"main process - {os.getpid()} taken - {end - start:.3f}s")
def run_multi_process():
    print("**** multi process ****")
    print(f"main process - {os.getpid()}")
    start = time.time()
    # p1 = Process(target=task_p, args=(1,))
    # p2 = Process(target=task_p, args=(2,))
    # p1.start()
    # p2.start()
    # p1.join()
    # p2.join()
    # for i in range(10):
    #     p = Process(target=task_p, args=(i,))
    #     p.start()
    #     p.join()
    processes = []
    for i in range(32):
        p = Process(target=task_p, args=(i,))
        p.start()
        processes.append(p)

    for p in processes:
        p.join()
    end = time.time()
    print(f"main process - {os.getpid()} taken - {end - start:.3f}s")
def task_t(x):
    time.sleep(2)
    print(f"Sub thread - {current_thread().name}: task - {x}: Result - {2**x})")
def run_multiple_threads():
    print("**** multiple threads ****")
    print(f"main process - {os.getpid()}")
    start = time.time()
    thread_p = []
    for i in range(32):
        t = threading.Thread(target=task_t, args=(i,))
        t.start()
        thread_p.append(t)
    for t in thread_p:
        t.join()

    end = time.time()
    print(f"main process - {os.getpid()} taken - {end - start:.3f}s")


if __name__ == "__main__":
    run_multi_process()
    run_multiple_threads()



import pybigtools
chrom_df = {
    "chr1": 248956422,
    "chr2": 242193529,
    "chr3": 198295559,
}
def covert_bigbed_to_bigwig(bigbed_file_path, output_file_path):
        def bigbed_to_bigwig(
                bigbed_file_path
        ):
            f = open(bigbed_file_path)
            for line in f:
                line_list = line.strip().split("\t")
                out_list = (line_list[0], int(line_list[1]), int(line_list[2]), float(line_list[3]))
                yield out_list
            f.close()

        b = pybigtools.open(output_file_path, "w")
        b.write(chrom_df, bigbed_to_bigwig(bigbed_file_path))

if __name__ == "__main__":
    covert_bigbed_to_bigwig("../data/293.ChIP.H3K36me3.rep1.ENCFF899GOH.bedGraph", "../data/test.bw")


import numpy as np
import bamnostic as bs
import pybigtools
# from my_deeptools_win import set_logging_level
# from rich.progress import track
import logging
from rich.logging import RichHandler
from rich.console import Console
FORMAT = "%(message)s"
logging.basicConfig(
    level="NOTSET", format=FORMAT, datefmt="[%X]", handlers=[RichHandler()]
)

log = logging.getLogger("rich")


def query_target_region_bin(
        bam_file_path,
        region_list,
        binsize = 10,
        scale_method = "cpm"
)-> np.array :
    """
    :param bam_obj: bam object
    :param region_list: list of (chr, start, end)
    :param binsize: bin size
    :return: numpy array of coverage
    """
    bam_obj = bs.AlignmentFile(bam_file_path, "rb")
    total_align_count = 0

    for bam_index_info in bam_obj.get_index_stats():
        # ref_name = bam_index_info[0]
        map_count = bam_index_info[0]

        total_align_count += map_count
    coverage = np.zeros(int((region_list[2] - region_list[1]) / binsize))
    chr_name = region_list[0]
    start, end = region_list[1], region_list[2]
    # print("chr:", chr_name)
    #print("start:", start)
    big_list = []
    for read in bam_obj.fetch(chr_name, start, end):
        coverage[int((read.reference_start-region_list[1])/binsize):int((read.reference_end-region_list[1]) /binsize)] += 1.0
    if scale_method == "count":
        coverage = coverage
    elif scale_method == "cpm":
        coverage = coverage / total_align_count * 1e6
    elif scale_method == "rpkm":
        coverage = coverage / (total_align_count * binsize) * 1e6

    #print(len(coverage))
    for i in range(coverage.shape[0]):
        start_in = start +  binsize
        big_list.append((chr_name, start, start_in, float(coverage[i])))
        start = start_in
    return big_list


# bs_file = bs.AlignmentFile("../data/293.ChIP.H3K36me3.rep1.ENCFF899GOH.bam","rb")
from multiprocessing import Pool
# chr_length_dt = {
#     "chr1": 248956422,
#     "chr2": 242193529,
#     "chr3": 198295559,
#     "chr4": 190214555,
#     "chr5": 181538259,
#     "chr6": 170805979,
#     "chr7": 159345973,
#     "chr8": 145138636,
# }
# chr_length_dt = {
#     "chr1": 248956422,
#     "chr2": 242193529,
#     "chr3": 198295559,
# }
# thread = 10
import time
def convert_bam_to_bigbed(bam_file_path, output_file_path,genome_dt,thread,binsize,scale_method):
    genome_file = open(genome_dt)
    chr_length_dt = {}
    for line in genome_file:
        line_list = line.strip().split("\t")
        chr_length_dt[line_list[0]] = int(line_list[1])
    genome_file.close()
    for index_num,chr_name in enumerate(chr_length_dt):
        log.info(f"start {chr_name}")
        start_time = time.time()

        pool = Pool(processes=thread)
        chr_bins = int(chr_length_dt[chr_name] / binsize)
        coverage_list = []
        for i in range(thread):
            coverage_list.append(pool.apply_async(query_target_region_bin, args=(
            bam_file_path,[chr_name, int(i * chr_bins / thread) * binsize, int((i + 1) * chr_bins / thread) * binsize],binsize,scale_method)))

        pool.close()
        pool.join()
        merge_total_count_list = []
        for temp_index, res in enumerate(coverage_list):
            run_res = res.get()
            # print(run_res)
            merge_total_count_list.append(run_res)
        if index_num == 0:
            f = open(output_file_path , "w")
        else:
            f = open(output_file_path , "a")
        for temp_index, res in enumerate(merge_total_count_list):
            # print(res)
            for temp_index, res in enumerate(res):
                # print(res)
                f.write(str(res[0]) + "\t" + str(res[1]) + "\t" + str(res[2]) + "\t" + str(res[3]) + "\n")
        # print(chr_name)
        # print(start_time)
        total_time = round((time.time() - start_time) / 60,2)
        log.info(f"convert {chr_name} time: {total_time}")
        # index_num += 1
        # print(f"convert {chr_name} time:", (time.time() - start_time) / 60)
        f.close()
def covert_bigbed_to_bigwig(bigbed_file_path, output_file_path,genome_dt):
    genome_file = open(genome_dt)
    chr_length_dt = {}
    for line in genome_file:
        line_list = line.strip().split("\t")
        chr_length_dt[line_list[0]] = int(line_list[1])
    genome_file.close()
    def bigbed_to_bigwig(
            bigbed_file_path
    ):
        f = open(bigbed_file_path)
        for line in f:
            line_list = line.strip().split("\t")
            out_list = (line_list[0], int(line_list[1]), int(line_list[2]), float(line_list[3]))
            yield out_list
        f.close()

    b = pybigtools.open(output_file_path, "w")
    b.write(chr_length_dt, bigbed_to_bigwig(bigbed_file_path))

def main(bam_file_path,
         bw_filw_path,
         genome_dt,
         tmp_path="./test_random.bigbed",
         threads=1,
         binsize=10,
         verbose="INFO",
         scale_method="cpm"):
    """

    :param bam_file_path:
    :param bw_filw_path:
    :param tmp_path:
    :param binsize:
    :param verbose:
    :return:
    """
    # set_logging_level(verbose)
    FORMAT = "%(message)s"
    logging.basicConfig(
        level=verbose, format=FORMAT, datefmt="[%X]", handlers=[RichHandler()]
    )

    log = logging.getLogger("rich")
    log.info("start calculation")
    #logging.info("start")
    console = Console()
    with console.status("[bold green]Working on tasks...") as status:
        convert_bam_to_bigbed(bam_file_path,tmp_path,genome_dt,thread=threads,binsize = binsize,scale_method=scale_method)
    log.info("start convert bigbed to bigwig")
    covert_bigbed_to_bigwig(tmp_path, bw_filw_path,genome_dt)
    import os
    os.remove(tmp_path)
    log.info("end")


if __name__ == "__main__":
    main("../data/293.ChIP.H3K36me3.rep1.ENCFF899GOH.bam",
         "../data/test.bw",
         genome_dt="../data/genome.fa.fai",
          tmp_path="../data/293.ChIP.H3K36me3.rep1.ENCFF899GOH.bedGraph",
         threads=16,
         scale_method="none")
    # logging.info("start count bam")
    # convert_bam_to_bigbed("../data/293.ChIP.H3K36me3.rep1.ENCFF899GOH.bam", "293.ChIP.H3K36me3.rep1.ENCFF899GOH.bedGraph")
    # logging.info("start convert bigbed to bw file")
    # covert_bigbed_to_bigwig("../data/293.ChIP.H3K36me3.rep1.ENCFF899GOH.bedGraph", "../data/293.ChIP.H3K36me3.rep1.ENCFF899GOH.bw")
    # logging.info("finish")
    # # for chr_name in chr_length_dt:
    # #     print(chr_name)
    # #     coverage = query_target_region_bin(bs_file, [(chr_name,0,chr_length_dt[chr_name])])
    # #     print(coverage)
    # start_time = time.time()
    # # 并行
    #
    # print("parallel")
    #
    # thread = 10
    # for chr_name in chr_length_dt:
    #     pool = Pool(processes=thread)
    #     #print(chr_name)
    #     chr_bins = int(chr_length_dt[chr_name] / 10)
    #     coverage_list = []
    #     for i in range(thread):
    #         coverage_list.append(pool.apply_async(query_target_region_bin, args=([chr_name,int(i*chr_bins/thread)*10,int((i+1)*chr_bins/thread)*10],)))
    #     #coverage_list.append(pool.apply_async(query_target_region_bin, args=([chr_name,0,chr_length_dt[chr_name]],)))
    #     # coverage = pool.apply_async(query_target_region_bin, args=(bs_file, [(chr_name,0,chr_length_dt[chr_name])]))
    #     # print(coverage)
    #     pool.close()
    #     pool.join()
    #     merge_total_count_list = []
    #     for temp_index, res in enumerate(coverage_list):
    #         run_res = res.get()
    #         # print(run_res)
    #         merge_total_count_list.append(run_res)
    #     f = open("../data/293.ChIP.H3K36me3.rep1.ENCFF899GOH.chr1.bedGraph", "w")
    #     for temp_index, res in enumerate(merge_total_count_list):
    #         # print(res)
    #         for temp_index, res in enumerate(res):
    #             # print(res)
    #             f.write(str(res[0]) + "\t" + str(res[1]) + "\t" + str(res[2]) + "\t" + str(res[3]) + "\n")
    #     print("time:", (time.time() - start_time)/60)
    #     f.close()
    # pool = Pool(processes=2)
    # coverage_list = []
    # for chr_name in chr_length_dt:
    #     print(chr_name)
    #     coverage_list.append(pool.apply_async(query_target_region_bin, args=(bs_file, [(chr_name,0,chr_length_dt[chr_name])])))
    # pool.close()
    # pool.join()
    # # for temp_index, res in enumerate(coverage_list):
    # #     run_res = res.get()
    # #     print(run_res)



import os
import logging
import random
import string
import gzip


def split_file_and_make_temp(input_file_name,
                             n_part = 1,
                             temp_dir=None):
    if temp_dir:
        temp_dir = os.path.abspath(temp_dir)
    else:
        temp_dir = os.path.dirname(input_file_name)

    if not os.path.exists(temp_dir):
        os.makedirs(temp_dir)
        logging.info(f"Create temp dir: {temp_dir}")
    input_file_basename = os.path.basename(input_file_name)

    logging.info(f"Split file: {input_file_basename}")
    temp_file_list = []
    temp_filename_list = []

    for index in range(n_part):
        rnd_str = ''.join(random.choices(string.ascii_letters + string.digits, k=16))
        temp_file_basename = f"temp_{input_file_basename}.{index}.{rnd_str}"
        #print(temp_file_basename)
        #logging.info(f"Create temp file: {temp_file_basename}")
        temp_file_name = os.path.join(temp_dir, temp_file_basename)
        temp_filename_list.append(temp_file_name)

        temp_file_list.append(open(temp_file_name,"wt"))

    logging.info("Counting input file")

    total_input_line_num = 0
    input_file = open(input_file_name,"rt") if not input_file_name.endswith(".gz") else gzip.open(input_file_name,"rt")
    for line in input_file:
        #print(line)
        total_input_line_num += 1
    input_file.close()
    # print(f"Total input line num: {total_input_line_num}")
    if total_input_line_num % n_part == 0:
        each_file_line_num = total_input_line_num // n_part
    else:
        each_file_line_num = total_input_line_num // n_part + 1

    logging.info("Start to split")

    input_file = open(input_file_name,"rt") if not input_file_name.endswith(".gz") else gzip.open(input_file_name,"rt")
    for index, line in enumerate(input_file):
        file_index = index // each_file_line_num
        temp_file_list[file_index].write(line)
    input_file.close()
    for temp_file in temp_file_list:
        temp_file.close()

    logging.info("Split done")
    return temp_filename_list



import pybigtools
import bamnostic as bs


def query_chrom_region_signal(
        signal_bam_obj,
        chr_name,
        chr_length,
        scale_method="rpkm",
        binsize=10
):
    """

    :param signal_bam_obj:
    :param chr_name:
    :param chr_length:
    :param scale_method:
    :param binsize:
    :return: generater (chr_name, start, end, signal_value)
    """
    total_count = 0
    for info in signal_bam_obj.get_index_stats():
        total_count += info[0]
    region_count_list = []
    for start in range(38526865, chr_length, binsize):
        end = min(start + binsize, chr_length)
        count = 0
        for read in signal_bam_obj.fetch(chr_name, start, end):
            count += 1
        if scale_method == "cpm":
            count = round(count / total_count * 1e6, 5)
        elif scale_method == "rpkm":
            region_length = end - start + 1
            count = round(count / region_length * 1e3, 5)
        print(end)
        yield (chr_name, start, end, count)

bs_file = bs.AlignmentFile("../data/K562-ATACSeq-rep1.ENCFF534DCE_chr21.bam","rb")
b = pybigtools.open("../data/test.bigWig", "w")
b.write({"chr21": 38530701}, iter(query_chrom_region_signal(bs_file, "chr21", 38530701)))